@@ -82,9 +82,10 @@ using namespace std;
 #define RENDER_STL_STR(str) \
 	oStream->Write (str.c_str(), str.size(), &bytesWriten);
 
-void ParseEnvPath(const nsCString &path, list<string> &dirs);
-int gHelpSelect (const struct dirent *dirEntry);
-int gnomeHelpSelect (const struct dirent *dirEntry);
+static void ParseEnvPath(const nsCString &path, list<string> &dirs);
+static int gHelpSelect (const struct dirent *dirEntry);
+static int gnomeHelpSelect (const struct dirent *dirEntry);
+static void RenderContentType (nsIOutputStream *oStream, PRUint32 &bytesWriten);
 
 /* Implementation file */
 NS_IMPL_ISUPPORTS1 (GTOCProtocolHandler, nsIProtocolHandler)
@@ -130,19 +131,9 @@ NS_METHOD GTOCProtocolHandler::CreatePage (void)
 	rv = mStream->NewInputStream(0, getter_AddRefs(iStream));
 	if (NS_FAILED(rv)) return rv;
 
-#if MOZILLA_SNAPSHOT > 4
         rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
                                       iStream, NS_LITERAL_CSTRING("text/html"),
                                       NS_LITERAL_CSTRING("utf-8"));
-#else
-	PRUint32 size;  
-	rv = mStream->GetLength(&size);
-	if (NS_FAILED(rv)) return rv;
-
-	rv = NS_NewInputStreamChannel(getter_AddRefs(mChannel), mURI,
-				      iStream, NS_LITERAL_CSTRING("text/html"),
-				      NS_LITERAL_CSTRING("utf-8"), size);
-#endif
 	if (NS_FAILED(rv)) return rv;
 
 	return rv;
@@ -166,19 +157,7 @@ NS_METHOD GTOCProtocolHandler::CreateTOCPage (void)
 	RENDER (_("GNOME Help Index"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-
-	if (!platformCharset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (NS_ConvertUCS2toUTF8(platformCharset).get());
-		RENDER ("\">");
-	}
+	RenderContentType (oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 
@@ -251,19 +230,7 @@ NS_METHOD GTOCProtocolHandler::CreateInfoPage (void)
 	RENDER (_("pages"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-
-	if (!platformCharset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (NS_ConvertUCS2toUTF8(platformCharset).get());
-		RENDER ("\">");
-	}
+	RenderContentType (oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 	RENDER ("<h3><strong>Info pages</strong></h3>\n");
@@ -334,19 +301,7 @@ NS_METHOD GTOCProtocolHandler::CreateManPage (void)
 	RENDER (_("pages"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-
-	if (!platformCharset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (NS_ConvertUCS2toUTF8(platformCharset).get());
-		RENDER ("\">");
-	}
+	RenderContentType (oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 	RENDER ("<h3><strong>Man pages</strong></h3>");
@@ -434,19 +389,7 @@ NS_METHOD GTOCProtocolHandler::CreateHelpPage (const char *type,
 	RENDER (_("pages"));
 	RENDER ("</title></head>\n");
 
-	nsCOMPtr<nsIPlatformCharset> pc =
-		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-
-	nsAutoString platformCharset;
-	rv = pc->GetCharset(kPlatformCharsetSel_Menu,
-			    platformCharset);
-
-	if (!platformCharset.IsEmpty())
-	{
-		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
-		RENDER (NS_ConvertUCS2toUTF8(platformCharset).get());
-		RENDER ("\">");
-	}
+	RenderContentType (oStream, bytesWriten);
 
 	RENDER("</head>\n<body>\n");
 	RENDER ("<h3><strong>Help pages</strong></h3>");
@@ -537,6 +480,31 @@ int gnomeHelpSelect (const struct dirent *dirEntry)
 #endif
 }
 
+void RenderContentType (nsIOutputStream *oStream, PRUint32 &bytesWriten)
+{
+	nsresult rv;
+	nsCOMPtr<nsIPlatformCharset> pc =
+		do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
+	g_return_if_fail (NS_SUCCEEDED(rv));
+	
+#if MOZILLA_SNAPSHOT > 9
+	nsCAutoString platformCharset;
+#else
+	nsAutoString platformCharset;
+#endif
+	rv = pc->GetCharset(kPlatformCharsetSel_Menu, platformCharset);
+	if (!platformCharset.IsEmpty())
+	{
+		RENDER ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=");
+#if MOZILLA_SNAPSHOT > 9
+		RENDER (platformCharset.get());
+#else
+		RENDER (NS_ConvertUCS2toUTF8(platformCharset).get());
+#endif
+		RENDER ("\">");
+	}
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 
