@@ -189,7 +189,7 @@ galeon_spinner_theme_changed (GConfClient *client,
 	gtk_widget_hide (GTK_WIDGET (spinner));
 	galeon_spinner_load_images (spinner);
 	gtk_widget_show (GTK_WIDGET (spinner));	
-	gtk_widget_queue_resize ( GTK_WIDGET (spinner));
+	gtk_widget_queue_resize (GTK_WIDGET (spinner));
 }
 
 static void
@@ -371,10 +371,8 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 
 	g_return_val_if_fail (IS_GALEON_SPINNER (widget), FALSE);
 
+	if (!GTK_WIDGET_DRAWABLE (widget)) return FALSE;
 	spinner = GALEON_SPINNER (widget);
-	if (!spinner->details->ready) {
-		return FALSE;
-	}
 
 	pixbuf = select_spinner_image (spinner);
 	if (pixbuf == NULL) {
@@ -397,8 +395,8 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 	height = gdk_pixbuf_get_height (pixbuf);
 
 	/* Compute the offsets for the image centered on our allocation */
-	x_offset = widget->allocation.x + (widget->allocation.width - width) / 2;
-	y_offset = widget->allocation.y + (widget->allocation.height - height) / 2;
+	x_offset = (widget->allocation.width - width) / 2;
+	y_offset = (widget->allocation.height - height) / 2;
 
 	pix_area.x = x_offset;
 	pix_area.y = y_offset;
@@ -424,18 +422,6 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 	return FALSE;
 }
 
-static void
-galeon_spinner_map (GtkWidget *widget)
-{
-	GaleonSpinner *spinner;
-	
-	spinner = GALEON_SPINNER (widget);
-
-	GTK_WIDGET_CLASS (parent_class)->map (widget);
-
-	spinner->details->ready = TRUE;
-}
-
 /* here's the actual timeout task to bump the frame and schedule a redraw */
 
 static gboolean 
@@ -444,9 +430,7 @@ bump_spinner_frame (gpointer callback_data)
 	GaleonSpinner *spinner;
 
 	spinner = GALEON_SPINNER (callback_data);
-	if (!spinner->details->ready) {
-		return TRUE;
-	}
+	if (!GTK_WIDGET_DRAWABLE (spinner)) return TRUE;
 
 	spinner->details->current_frame += 1;
 	if (spinner->details->current_frame > spinner->details->max_frame - 1) {
@@ -617,6 +601,7 @@ galeon_spinner_load_images (GaleonSpinner *spinner)
 	spinner->details->image_list = g_list_reverse (image_list);
 
 	g_free (image_theme);
+	g_free (path);
 }
 
 static gboolean
@@ -759,7 +744,6 @@ galeon_spinner_class_init (GaleonSpinnerClass *class)
 	widget_class->enter_notify_event = galeon_spinner_enter_notify_event;
 	widget_class->leave_notify_event = galeon_spinner_leave_notify_event;
 	widget_class->size_request = galeon_spinner_size_request;	
-	widget_class->map = galeon_spinner_map;
 }
 
 static void
@@ -774,7 +758,7 @@ galeon_spinner_search_directory  (const gchar *base, GList **spinner_list)
 			       GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
 	if (rc != GNOME_VFS_OK) return;
 
-	for (node = list; node != NULL; node = g_list_next (node))
+	for (node = list; node != NULL; node = node->next)
 	{
 		GnomeVFSFileInfo *file_info = node->data;
 		GaleonSpinnerInfo *info;
@@ -857,7 +841,7 @@ galeon_spinner_list_spinners (void)
 	GList *spinner_list = NULL;
 	GList *tmp;
 
-	for (tmp = spinner_directories; tmp != NULL; tmp = g_list_next (tmp))
+	for (tmp = spinner_directories; tmp != NULL; tmp = tmp->next)
 	{
 		gchar *path = tmp->data;
 		galeon_spinner_search_directory (path, &spinner_list);
@@ -872,7 +856,7 @@ galeon_spinner_get_theme_path (const gchar *theme_name)
 	GaleonSpinnerInfo *info;
 	GList *tmp;
 
-	for (tmp = spinner_directories; tmp != NULL; tmp = g_list_next (tmp))
+	for (tmp = spinner_directories; tmp != NULL; tmp = tmp->next)
 	{
 		gchar *path = tmp->data;
 
