@@ -25,6 +25,8 @@
 #include "gul-state.h"
 #include "gul-gobject-misc.h"
 #include "gul-notebook.h"
+#include "gul-toolbar.h"
+#include "gul-gui.h"
 #include "statusbar.h"
 #include "toolbar.h"
 #include "ppview-toolbar.h"
@@ -63,23 +65,35 @@
 #define GO_BACK_CMD_PATH "/commands/GoBack"
 #define GO_FORWARD_CMD_PATH "/commands/GoForward"
 #define GO_UP_CMD_PATH "/commands/GoUp"
+#define GO_STOP_CMD_PATH "/commands/GoStop"
 #define EDIT_FIND_NEXT_CMD_PATH "/commands/EditFindNext"
 #define EDIT_FIND_PREV_CMD_PATH "/commands/EditFindPrev"
 #define VIEW_SIDEBAR_PATH "/commands/View Sidebar"
 #define VIEW_MENUBAR_PATH "/commands/View Menubar"
 #define VIEW_STATUSBAR_PATH "/commands/View Statusbar"
 #define VIEW_TOOLBAR_PATH "/commands/View Toolbar"
-#define VIEW_BOOKMARKSBAR_PATH "/commands/View BookmarksBar"
+#define VIEW_BOOKMARKS_TOOLBAR_PATH "/commands/View Bookmarks Toolbars"
 #define VIEW_FULLSCREEN_PATH "/commands/View Fullscreen"
-#define ALLOW_POPUPS_PATH "/commands/Allow Popups"
+#define ALLOW_POPUPS_TOGGLE_PATH "/commands/Allow Popups"
 #define ALLOW_JAVA_PATH "/commands/Allow Java"
 #define ALLOW_JAVASCRIPT_PATH "/commands/Allow JavaScript"
+#define ALLOW_POPUPS_CMD_PATH "/commands/AllowPopupSite"
+#define ALLOW_COOKIES_CMD_PATH "/commands/AllowCookieSite"
+#define BLOCK_COOKIES_CMD_PATH "/commands/BlockCookieSite"
+#define ALLOW_IMAGES_CMD_PATH "/commands/AllowImageSite"
+#define BLOCK_IMAGES_CMD_PATH "/commands/BlockImageSite"
+#define ALLOW_POPUPS_MENU_PATH "/menu/Web/Popups/AllowPopupSite"
+#define ALLOW_COOKIES_MENU_PATH "/menu/Web/Cookies/AllowCookieSite"
+#define BLOCK_COOKIES_MENU_PATH "/menu/Web/Cookies/BlockCookieSite"
+#define ALLOW_IMAGES_MENU_PATH "/menu/View/Images/AllowImageSite"
+#define BLOCK_IMAGES_MENU_PATH "/menu/View/Images/BlockImageSite"
+
 
 #define ID_VIEW_SIDEBAR "View Sidebar"
 #define ID_VIEW_MENUBAR "View Menubar"
 #define ID_VIEW_STATUSBAR "View Statusbar"
 #define ID_VIEW_TOOLBAR "View Toolbar"
-#define ID_VIEW_BOOKMARKSBAR "View BookmarksBar"
+#define ID_VIEW_BOOKMARKS_TOOLBAR "View Bookmarks Toolbars"
 #define ID_VIEW_FULLSCREEN "View Fullscreen"
 #define ID_ALLOW_POPUPS "Allow Popups"
 #define ID_ALLOW_JAVA "Allow Java"
@@ -114,9 +128,8 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("FileNewTab", (BonoboUIVerbFn)window_cmd_new_tab),
 	BONOBO_UI_VERB ("FileOpen", (BonoboUIVerbFn)window_cmd_file_open),
 	BONOBO_UI_VERB ("FileSaveAs", (BonoboUIVerbFn)window_cmd_file_save_as),
+	BONOBO_UI_VERB ("FileQuit", (BonoboUIVerbFn)window_cmd_file_quit),
 	BONOBO_UI_VERB ("FileCloseTab", (BonoboUIVerbFn)window_cmd_file_close_tab),
-	BONOBO_UI_VERB ("FileCloseWindow", (BonoboUIVerbFn)window_cmd_file_close_window),
-	BONOBO_UI_VERB ("FileExit", (BonoboUIVerbFn)window_cmd_file_exit),
 	BONOBO_UI_VERB ("FileSendTo", (BonoboUIVerbFn)window_cmd_file_send_to),
 	BONOBO_UI_VERB ("EditCut", (BonoboUIVerbFn)window_cmd_edit_cut),
 	BONOBO_UI_VERB ("EditCopy", (BonoboUIVerbFn)window_cmd_edit_copy),
@@ -130,8 +143,11 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("Zoom Out", (BonoboUIVerbFn)window_cmd_view_zoom_out),
 	BONOBO_UI_VERB ("Zoom Normal", (BonoboUIVerbFn)window_cmd_view_zoom_normal),
 	BONOBO_UI_VERB ("ViewPageSource", (BonoboUIVerbFn)window_cmd_view_page_source),
-//	BONOBO_UI_VERB ("BlockCookieSite", (BonoboUIVerbFn)window_cmd_set_permission),
-//	BONOBO_UI_VERB ("AllowCookieSite", (BonoboUIVerbFn)window_cmd_set_permission),
+	BONOBO_UI_VERB ("AllowPopupSite", (BonoboUIVerbFn)window_cmd_set_permission),
+	BONOBO_UI_VERB ("BlockImageSite", (BonoboUIVerbFn)window_cmd_set_permission),
+	BONOBO_UI_VERB ("AllowImageSite", (BonoboUIVerbFn)window_cmd_set_permission),
+	BONOBO_UI_VERB ("BlockCookieSite", (BonoboUIVerbFn)window_cmd_set_permission),
+	BONOBO_UI_VERB ("AllowCookieSite", (BonoboUIVerbFn)window_cmd_set_permission),
 	BONOBO_UI_VERB ("ShowJavaConsole", (BonoboUIVerbFn)window_cmd_show_java_console),
 	BONOBO_UI_VERB ("ShowJSConsole", (BonoboUIVerbFn)window_cmd_show_js_console),
 	BONOBO_UI_VERB ("BookmarksAddDefault", (BonoboUIVerbFn)window_cmd_bookmarks_add_default),
@@ -143,6 +159,7 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("TabsMoveLeft", (BonoboUIVerbFn)window_cmd_tabs_move_left),
 	BONOBO_UI_VERB ("TabsMoveRight", (BonoboUIVerbFn)window_cmd_tabs_move_right),
 	BONOBO_UI_VERB ("TabsDetach", (BonoboUIVerbFn)window_cmd_tabs_detach),
+	BONOBO_UI_VERB ("TabsClone", (BonoboUIVerbFn)window_cmd_tabs_clone),
 	BONOBO_UI_VERB ("HelpContents", (BonoboUIVerbFn)window_cmd_help_manual),
 	BONOBO_UI_VERB ("About", (BonoboUIVerbFn)window_cmd_help_about),
 
@@ -155,6 +172,8 @@ static BonoboUIVerb galeon_popup_verbs [] = {
 	BONOBO_UI_VERB ("EPAddBookmark", (BonoboUIVerbFn)popup_cmd_add_bookmark),
 	BONOBO_UI_VERB ("EPOpenImageInNewWindow", (BonoboUIVerbFn)popup_cmd_image_in_new_window),
 	BONOBO_UI_VERB ("EPOpenImageInNewTab", (BonoboUIVerbFn)popup_cmd_image_in_new_tab),
+	BONOBO_UI_VERB ("EPBlockImageSite", (BonoboUIVerbFn)popup_cmd_image_block_site),
+	BONOBO_UI_VERB ("EPAllowImageSite", (BonoboUIVerbFn)popup_cmd_image_block_site),
 	BONOBO_UI_VERB ("DPOpenFrameInNewWindow", (BonoboUIVerbFn)popup_cmd_frame_in_new_window),
 	BONOBO_UI_VERB ("DPOpenFrameInNewTab", (BonoboUIVerbFn)popup_cmd_frame_in_new_tab),
 	BONOBO_UI_VERB ("DPAddFrameBookmark", (BonoboUIVerbFn)popup_cmd_add_frame_bookmark),
@@ -224,6 +243,11 @@ static void
 galeon_window_recent_history_menu_activated_cb (GaleonWindowRecentHistoryMenu *wrhm, 
 						const char *url, const char *title, 
 						GaleonWindow *w);
+static void
+galeon_window_fullscreen (GaleonWindow *window);
+static void
+galeon_window_unfullscreen (GaleonWindow *window);
+
 
 /* static class variables */
 static GdkColor _galeon_window_loading_tab_color;
@@ -273,10 +297,10 @@ galeon_window_key_press_event_cb (GtkWidget *widget,
 {
         int page;
 
-	if ((event->state & GDK_Shift_L) || (event->state & GDK_Shift_R))
+	if (event->state & GDK_SHIFT_MASK)
                 return FALSE;
 
-        if ((event->state & GDK_Alt_L) || (event->state & GDK_Alt_R))
+        if (event->state & GDK_MOD1_MASK)
         {
                 page = event->keyval - GDK_0 -1;
 
@@ -300,13 +324,23 @@ galeon_window_selection_received_cb (GtkWidget *widget,
 				     guint time, GaleonWindow *window)
 {
 	GaleonTab *tab;
+	gint action;
 
 	if (selection_data->length <= 0 || selection_data->data == NULL)
 		return;
 
-	tab = galeon_window_get_active_tab (window);
-	galeon_shell_new_tab (galeon_shell, window, tab,
-			      selection_data->data, 0);
+	tab = galeon_window_get_active_tab(window);
+	action = eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION);
+	
+	if (action == 6) //Paste into current tab
+	{
+		galeon_window_load_url(window, selection_data->data);
+	}
+	else //Paste into new tab (action == 1 or default)
+	{
+		galeon_shell_new_tab(galeon_shell, window, tab,
+ 				     selection_data->data, 0);
+	}
 }
 	
 static void
@@ -335,10 +369,29 @@ galeon_window_state_event_cb (GtkWidget *widget,
 			      GdkEventWindowState *event,
 			      gpointer data)
 {
+	if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
+	{
+		GaleonWindow *window = GALEON_WINDOW (widget);
+		if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN)
+		{
+			galeon_window_fullscreen (window);
+		}
+		else
+		{
+			galeon_window_unfullscreen (window);
+		}
+	}
 	save_window_state (widget);
 	return FALSE;
 }
 
+static gboolean
+delete_event_cb (GaleonWindow *window)
+{
+	window->priv->closing = TRUE;
+	return gul_gui_widget_hide_now (GTK_WIDGET(window));
+}
+
 static void
 setup_bonobo_window (GaleonWindow *window,
 		     BonoboUIComponent **ui_component)
@@ -362,11 +415,7 @@ setup_bonobo_window (GaleonWindow *window,
 
 	bonobo_ui_util_set_ui (*ui_component,
 			       DATADIR,
-#ifdef ENABLE_SIMPLE_MENUBAR
-			       "galeon-simple-ui.xml",
-#else
 			       "galeon-ui.xml",
-#endif
 			       "galeon", NULL);
 
 	/* Key handling */
@@ -379,6 +428,8 @@ setup_bonobo_window (GaleonWindow *window,
 			  G_CALLBACK (galeon_window_configure_event_cb), NULL);
 	g_signal_connect (G_OBJECT(win), "window_state_event",
 			  G_CALLBACK (galeon_window_state_event_cb), NULL);
+	g_signal_connect (G_OBJECT(win), "delete-event",
+			  G_CALLBACK (delete_event_cb), NULL);
 	
 	g_signal_connect (G_OBJECT(win),
 			  "selection-received",
@@ -411,11 +462,22 @@ setup_popup_factory (GaleonWindow *window,
 	return popup;
 }
 
+static void
+tabbed_position_gconf_changed_cb(GConfClient *client,
+                             guint cnxn_id,
+                             GConfEntry *entry,
+                             GtkNotebook *notebook)
+{
+	GtkPositionType position = eel_gconf_get_integer (CONF_TABS_TABBED_EDGE);    
+	gtk_notebook_set_tab_pos (notebook, position);
+}
+
 static GtkNotebook *
 setup_notebook (GaleonWindow *window)
 {
 	GtkNotebook *notebook;
 	GtkPolicyType policy;
+	GtkPositionType position = eel_gconf_get_integer (CONF_TABS_TABBED_EDGE);
 
 	policy = eel_gconf_get_boolean (CONF_TABS_TABBED_ALWAYS_SHOW)
 		 ? GTK_POLICY_ALWAYS
@@ -424,6 +486,7 @@ setup_notebook (GaleonWindow *window)
 	notebook = GTK_NOTEBOOK (gul_notebook_new ());
 	gtk_notebook_set_scrollable (notebook, TRUE);
 	gtk_notebook_set_show_border (notebook, FALSE);
+	gtk_notebook_set_tab_pos (notebook, position);
 	gul_notebook_set_policy (GUL_NOTEBOOK (notebook), policy);
 	gtk_notebook_popup_enable (notebook);
 
@@ -436,6 +499,10 @@ setup_notebook (GaleonWindow *window)
 			  G_CALLBACK (galeon_window_tab_detached_cb), 
 			  window);
 
+	eel_gconf_notification_add(CONF_TABS_TABBED_EDGE,
+				   (GConfClientNotifyFunc)tabbed_position_gconf_changed_cb,
+				   notebook);
+    
 	gtk_widget_show (GTK_WIDGET (notebook));
 
 	return notebook;
@@ -504,6 +571,22 @@ view_toolbar_changed_cb (BonoboUIComponent *component,
         galeon_window_set_chrome (window, mask);
 }
 
+static void
+view_bookmarks_toolbar_changed_cb (BonoboUIComponent *component,
+				   const char *path,
+				   Bonobo_UIComponent_EventType type,
+				   const char *state,
+				   GaleonWindow *window)
+{
+	EmbedChromeMask mask;
+	
+	if (window->priv->ignore_layout_toggles) return;
+
+        mask = galeon_window_get_chrome (window);
+        mask ^= EMBED_CHROME_PERSONALTOOLBARON;
+        galeon_window_set_chrome (window, mask);
+}
+
 static void
 view_sidebar_changed_cb (BonoboUIComponent *component,
                          const char *path,
@@ -543,14 +626,18 @@ view_fullscreen_changed_cb (BonoboUIComponent *component,
                             const char *state,
                             GaleonWindow *window)
 {	
-	EmbedChromeMask mask;
-	
 	if (window->priv->ignore_layout_toggles) return;
-	
-	mask = galeon_window_get_chrome (window);
-	mask ^= EMBED_CHROME_OPENASFULLSCREEN;
-	mask |= EMBED_CHROME_DEFAULT;
-	galeon_window_set_chrome (window, mask);
+
+	if (state == NULL || *state == '\0') return;
+
+	if (strcmp (state, "1") == 0)
+	{
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	}
+	else
+	{
+		gtk_window_unfullscreen (GTK_WINDOW (window));
+	}
 }
 
 static void
@@ -565,10 +652,15 @@ update_layout_toggles(GaleonWindow *window)
 				    mask & EMBED_CHROME_SIDEBARON);
 	gul_bonobo_set_toggle_state(ui_component, VIEW_TOOLBAR_PATH, 
 				    mask & EMBED_CHROME_TOOLBARON);
+	gul_bonobo_set_toggle_state(ui_component, VIEW_BOOKMARKS_TOOLBAR_PATH, 
+				    mask & EMBED_CHROME_PERSONALTOOLBARON);
 	gul_bonobo_set_toggle_state(ui_component, VIEW_STATUSBAR_PATH, 
 				    mask & EMBED_CHROME_STATUSBARON);
 	gul_bonobo_set_toggle_state(ui_component, VIEW_FULLSCREEN_PATH, 
 				    mask & EMBED_CHROME_OPENASFULLSCREEN);
+	toolbar_set_fullscreen (window->priv->toolbar,
+				mask & EMBED_CHROME_OPENASFULLSCREEN);
+
 
 	window->priv->ignore_layout_toggles = FALSE;
 }
@@ -584,6 +676,9 @@ setup_layout_menus(GaleonWindow *window)
 	bonobo_ui_component_add_listener(ui_component, ID_VIEW_TOOLBAR, 
 					 (BonoboUIListenerFn)view_toolbar_changed_cb, 
 					 window);
+	bonobo_ui_component_add_listener(ui_component, ID_VIEW_BOOKMARKS_TOOLBAR, 
+					 (BonoboUIListenerFn)view_bookmarks_toolbar_changed_cb, 
+					 window);
 	bonobo_ui_component_add_listener(ui_component, ID_VIEW_STATUSBAR, 
 					 (BonoboUIListenerFn)view_statusbar_changed_cb, 
 					 window);
@@ -653,12 +748,34 @@ allow_setting_gconf_changed_cb(GConfClient *client,
 			       GConfEntry *entry,
 			       BonoboUIComponent *ui_component)
 {
-	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_PATH,
-				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_TOGGLE_PATH,
+				    eel_gconf_get_boolean(CONF_FILTERING_ALLOW_POPUPS));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVASCRIPT_PATH,
-				    eel_gconf_get_boolean (CONF_FILTERING_JAVASCRIPT_ENABLED));
+				    eel_gconf_get_boolean(CONF_FILTERING_JAVASCRIPT_ENABLED));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVA_PATH,
-				    eel_gconf_get_boolean (CONF_FILTERING_JAVA_ENABLED));
+				    eel_gconf_get_boolean(CONF_FILTERING_JAVA_ENABLED));
+}
+
+//Prototype needed as implemented later
+static void update_permission_control(GaleonWindow *window);
+
+static void
+allow_popups_bonoboui_changed_cb(BonoboUIComponent *component,
+				 const char *path,
+				 Bonobo_UIComponent_EventType type,
+				 const char *state,
+				 GaleonWindow *window)
+{
+	update_permission_control(window);
+}
+
+static void
+allow_popups_gconf_changed_cb(GConfClient *client,
+			      guint cnxn_id,
+			      GConfEntry *entry,
+			      GaleonWindow *window)
+{
+	update_permission_control(window);
 }
 
 static void
@@ -689,10 +806,15 @@ accept_cookies_gconf_changed_cb(GConfClient *client,
 static void
 setup_web_menu (GaleonWindow *window)
 {
+	GConfValue *gcvalue;
+	
 	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
 
-	radio_menu_group_set_from_value(ui_component, COOKIES,
-					eel_gconf_get_value(CONF_PERSISTENT_COOKIES_BEHAVIOR));
+	gcvalue = eel_gconf_get_value(CONF_PERSISTENT_COOKIES_BEHAVIOR);
+
+	radio_menu_group_set_from_value(ui_component, COOKIES, gcvalue);
+
+	gconf_value_free (gcvalue);
 
 	bonobo_ui_component_add_listener(ui_component, "AcceptCookiesAlways",
 					 accept_cookies_bonoboui_changed_cb,
@@ -708,13 +830,16 @@ setup_web_menu (GaleonWindow *window)
 				   (GConfClientNotifyFunc)accept_cookies_gconf_changed_cb, 
 				   ui_component);
 
-	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_PATH,
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_TOGGLE_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVASCRIPT_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_JAVASCRIPT_ENABLED));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVA_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_JAVA_ENABLED));
 
+	bonobo_ui_component_add_listener(ui_component, ID_ALLOW_POPUPS,
+					 (BonoboUIListenerFn)allow_popups_bonoboui_changed_cb,
+					 window);
 	bonobo_ui_component_add_listener(ui_component, ID_ALLOW_POPUPS,
 					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
 					 CONF_FILTERING_ALLOW_POPUPS);
@@ -725,6 +850,9 @@ setup_web_menu (GaleonWindow *window)
 					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
 					 CONF_FILTERING_JAVA_ENABLED);
 
+	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
+				   (GConfClientNotifyFunc)allow_popups_gconf_changed_cb,
+				   window);
 	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
 				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
 				   ui_component);
@@ -764,10 +892,13 @@ view_images_gconf_changed_cb(GConfClient *client,
 static void
 setup_images_menu(GaleonWindow *window)
 {
+	GConfValue *gcvalue;
 	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT(window->ui_component);
 
-	radio_menu_group_set_from_value(ui_component, IMAGES,
-					eel_gconf_get_value(CONF_FILTERING_IMAGE_LOADING_TYPE));
+	gcvalue = eel_gconf_get_value(CONF_FILTERING_IMAGE_LOADING_TYPE);
+	radio_menu_group_set_from_value(ui_component, IMAGES, gcvalue);
+
+	gconf_value_free (gcvalue);
 
 	bonobo_ui_component_add_listener(ui_component, "ViewImagesAlways",
 					 view_images_bonoboui_changed_cb,
@@ -789,9 +920,12 @@ sidebar_close_requested_cb (GaleonSidebar *sidebar,
 			    GaleonWindow *window)
 {
 	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
-	
-	gul_bonobo_set_toggle_state (ui_component, VIEW_SIDEBAR_PATH, 
-				     FALSE);
+
+	view_sidebar_changed_cb (ui_component,
+		VIEW_SIDEBAR_PATH,
+		BONOBO_TYPE_UI_COMPONENT,
+		NULL,
+		window);
 }
 
 static void
@@ -849,7 +983,7 @@ setup_sidebar (GaleonWindow *window)
 	sidebar = galeon_sidebar_new ();
 
 	g_signal_connect (G_OBJECT(sidebar),
-			  "close-requested",
+			  "close_requested",
 			  G_CALLBACK (sidebar_close_requested_cb),
 			  window);
 	
@@ -904,7 +1038,7 @@ galeon_window_init (GaleonWindow *window)
 						     window);
 	setup_layout_menus (window);
 	setup_images_menu (window);
-	setup_web_menu(window);
+	setup_web_menu (window);
 	
 	/* Setup the embed popups factory */
 	window->priv->embed_popup = setup_popup_factory (window, 
@@ -962,6 +1096,8 @@ save_window_chrome (GaleonWindow *window)
 				       flags & EMBED_CHROME_SIDEBARON);
 		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_TOOLBARS,
 				       flags & EMBED_CHROME_TOOLBARON);
+		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_BOOKMARKS,
+				       flags & EMBED_CHROME_PERSONALTOOLBARON);
 		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_STATUSBAR,
 				       flags & EMBED_CHROME_STATUSBARON);
 	}
@@ -971,8 +1107,10 @@ save_window_chrome (GaleonWindow *window)
 				       flags & EMBED_CHROME_SIDEBARON);
 		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_TOOLBARS,
 				       flags & EMBED_CHROME_TOOLBARON);
+		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_BOOKMARKS,
+				       flags & EMBED_CHROME_PERSONALTOOLBARON);
 		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_STATUSBAR,
-				       flags & EMBED_CHROME_STATUSBARON);		
+				       flags & EMBED_CHROME_STATUSBARON);
 	}
 }
 
@@ -1067,57 +1205,6 @@ galeon_window_get_chrome (GaleonWindow *window)
 	return window->priv->chrome_mask;
 }
 
-static void
-wmspec_change_state (gboolean   add,
-                     GdkWindow *window,
-                     GdkAtom    state1,
-                     GdkAtom    state2)
-{
-	XEvent xev;
-
-	#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
-	#define _NET_WM_STATE_ADD           1    /* add/set property */
-	#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
-  
-	xev.xclient.type = ClientMessage;
-  	xev.xclient.serial = 0;
-  	xev.xclient.send_event = True;
-  	xev.xclient.display = gdk_display;
-  	xev.xclient.window = GDK_WINDOW_XID (window);
-  	xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_WM_STATE");
-  	xev.xclient.format = 32;
-  	xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
-  	xev.xclient.data.l[1] = gdk_x11_atom_to_xatom (state1);
-  	xev.xclient.data.l[2] = gdk_x11_atom_to_xatom (state2);
-  
-  	XSendEvent (gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
-                    False,
-                    SubstructureRedirectMask | SubstructureNotifyMask,
-                    &xev);
-}
-
-static void
-window_set_fullscreen_mode (GaleonWindow *window, gboolean active)
-{
-        GdkWindow *gdk_window;
-
-        gdk_window = GTK_WIDGET(window)->window;
-
-        if (gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
-                                                  FALSE)))
-        {
-                wmspec_change_state (active,
-                                     gdk_window,
-                                     gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
-                                                      FALSE),
-                                     GDK_NONE);
-        }
-        else
-        {
-                g_warning ("NET_WM_STATE_FULLSCREEN not supported");
-	}
-}
-
 static void
 translate_default_chrome (EmbedChromeMask *chrome_mask)
 {	
@@ -1164,12 +1251,43 @@ translate_default_chrome (EmbedChromeMask *chrome_mask)
 		{
 			*chrome_mask |= EMBED_CHROME_TOOLBARON;
 		}
+		if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS))
+		{
+			*chrome_mask |= EMBED_CHROME_PERSONALTOOLBARON;
+		}
 		
-		*chrome_mask |= EMBED_CHROME_PERSONALTOOLBARON;
 		*chrome_mask |= EMBED_CHROME_MENUBARON;
 	}
 }
 
+/* Called after GaleonWindow has switched to fullscreen, setup toolbars etc. to
+ * match.
+ */
+static void
+galeon_window_fullscreen (GaleonWindow *window)
+{
+	EmbedChromeMask mask;
+
+	mask = window->priv->chrome_mask;
+	mask |= EMBED_CHROME_DEFAULT;
+	mask |= EMBED_CHROME_OPENASFULLSCREEN;
+	galeon_window_set_chrome (window, mask);
+}
+
+/* Called after GaleonWindow has switched back to normal size again, restore
+ * normal toolbar etc. layouts.
+ */
+static void
+galeon_window_unfullscreen (GaleonWindow *window)
+{
+	EmbedChromeMask mask;
+
+	mask = window->priv->chrome_mask;
+	mask |= EMBED_CHROME_DEFAULT;
+	mask &= ~EMBED_CHROME_OPENASFULLSCREEN;
+	galeon_window_set_chrome (window, mask);
+}
+
 void  
 galeon_window_set_chrome (GaleonWindow *window, 
 			  EmbedChromeMask flags)
@@ -1207,15 +1325,6 @@ galeon_window_set_chrome (GaleonWindow *window,
 	ppview_toolbar_set_visibility (window->priv->ppview_toolbar, 
 				       ppvtoolbar);
 	
-	/* set fullscreen only when it's really changed */
-	if ((window->priv->chrome_mask & EMBED_CHROME_OPENASFULLSCREEN) != 
-	    (flags & EMBED_CHROME_OPENASFULLSCREEN))
-	{
-		save_window_chrome (window);
-		window_set_fullscreen_mode (window,
-					    flags & EMBED_CHROME_OPENASFULLSCREEN);
-	}
-
 	gb_bonobo_ui_toolbar_set_visible (window->priv->bookmarks_toolbar,
 					  flags & EMBED_CHROME_PERSONALTOOLBARON);
 	
@@ -1226,6 +1335,19 @@ galeon_window_set_chrome (GaleonWindow *window,
 	save_window_chrome (window);
 }
 
+void
+galeon_window_toggle_fullscreen (GaleonWindow *window)
+{
+	if (gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_FULLSCREEN)
+	{
+		gtk_window_unfullscreen (GTK_WINDOW (window));
+	}
+	else
+	{
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	}
+}
+
 void 
 galeon_window_add_tab (GaleonWindow *window, 
 		       GaleonTab *tab,
@@ -1246,6 +1368,18 @@ galeon_window_add_tab (GaleonWindow *window,
 				  jump_to);
 }
 
+void
+galeon_window_next_tab (GaleonWindow *window)
+{
+	gtk_notebook_next_page (GTK_NOTEBOOK(window->priv->notebook));
+}
+
+void
+galeon_window_prev_tab (GaleonWindow *window)
+{
+	gtk_notebook_prev_page (GTK_NOTEBOOK(window->priv->notebook));
+}
+
 void 
 galeon_window_jump_to_tab (GaleonWindow *window, 
 			   GaleonTab *tab)
@@ -1298,8 +1432,6 @@ galeon_window_remove_tab (GaleonWindow *window,
 	g_return_if_fail (IS_GALEON_WINDOW (window));
 	g_return_if_fail (IS_GALEON_TAB (tab));
 
-	window->priv->active_tab = NULL;
-	
 	embed  = GTK_WIDGET (galeon_tab_get_embed (tab));
 	
 	gul_notebook_remove_page (GUL_NOTEBOOK (window->priv->notebook), 
@@ -1340,27 +1472,16 @@ galeon_window_load_url (GaleonWindow *window,
         g_return_if_fail (url != NULL);
 	
 	set = galeon_shell_get_bookmark_set (galeon_shell);
-	if (set)
-	{
-		real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
-		if (!real_url)
-		{
-			/* no matching nick found */
-			real_url = g_strdup (url);
-		}
-	}
-	else
-	{
-		real_url = g_strdup (url);
-	}
+	real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
+
+        galeon_embed_load_url (embed, real_url != NULL ? real_url : url);
 
-        galeon_embed_load_url (embed, real_url);
 	g_free (real_url);
 }
 
-void galeon_window_activate_location (GaleonWindow *window)
+void galeon_window_edit_location (GaleonWindow *window)
 {
-	toolbar_activate_location (window->priv->toolbar);
+	toolbar_edit_location (window->priv->toolbar);
 }
 
 void
@@ -1382,7 +1503,7 @@ galeon_window_show (GtkWidget *widget)
 	{
 		gul_state_load_window (GTK_WIDGET(window), 
 				       "main_window", 
-			               600, 500, TRUE);
+			               700, 500, TRUE);
 		window->priv->has_default_size = TRUE;
 	}
 	
@@ -1485,6 +1606,21 @@ update_security (GaleonWindow *window)
 	g_free (tooltip);
 }
 
+static void
+update_popup_block(GaleonWindow *window)
+{
+	GaleonTab *tab;
+	const char *uri = NULL;
+
+	tab = galeon_window_get_active_tab (window);
+	g_return_if_fail (tab != NULL);
+
+	uri = galeon_tab_get_blocked_popup_uri(tab);
+
+	statusbar_set_popup_block(STATUSBAR(window->priv->statusbar),
+				  uri ? TRUE : FALSE, uri);
+}
+
 static void
 update_nav_control (GaleonWindow *window)
 {
@@ -1533,6 +1669,8 @@ update_nav_control (GaleonWindow *window)
 			          GO_FORWARD_CMD_PATH, !forward);
 	gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
 			          GO_UP_CMD_PATH, !up);
+	gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
+			          GO_STOP_CMD_PATH, stop);
 }
 
 static void
@@ -1557,17 +1695,63 @@ static void
 update_location_control (GaleonWindow *window)
 {
 	GaleonTab *tab;
-	const char *location;
-	
+	const char *location, *host = NULL;
+	char *allow_images_str, *block_images_str,
+	     *allow_cookies_str, *block_cookies_str,
+	     *allow_popups_str;
+	GnomeVFSURI *uri = NULL;
+
 	tab = galeon_window_get_active_tab (window);
 	g_return_if_fail (tab != NULL);
 
 	location = galeon_tab_get_location (tab);
-	
-	if (!location) location = "";
+
+	/* don't display about:blank in the location entry, it's uninteresting
+	 * and prevents the user from pasting a new url after opening a new
+	 * window/tab */
+	if (!location || !strcmp (location, "about:blank")) location = "";
 
 	toolbar_set_location (window->priv->toolbar,
 			      location);
+
+	/* Update cookie and image allow/block menu items to refer to
+	 * the current host. Makes things clearer for the user
+	 */
+	if (location)
+		uri = gnome_vfs_uri_new(location);
+
+	if (uri)
+		host = gnome_vfs_uri_get_host_name(uri);
+
+
+	if (!host)
+		host = _("This Site");
+
+	allow_cookies_str = g_strdup_printf(_("_Allow Cookies From %s"), host);
+	block_cookies_str = g_strdup_printf(_("_Block Cookies From %s"), host);
+	allow_images_str = g_strdup_printf(_("_Allow Images From %s"), host);
+	block_images_str = g_strdup_printf(_("_Block Images From %s"), host);
+	allow_popups_str = g_strdup_printf(_("_Allow Popups From %s"), host);
+
+	if (uri)
+		gnome_vfs_uri_unref(uri);
+
+	gul_bonobo_set_label(BONOBO_UI_COMPONENT(window->ui_component),
+			     ALLOW_COOKIES_MENU_PATH, allow_cookies_str);
+	gul_bonobo_set_label(BONOBO_UI_COMPONENT(window->ui_component),
+			     BLOCK_COOKIES_MENU_PATH, block_cookies_str);
+	gul_bonobo_set_label(BONOBO_UI_COMPONENT(window->ui_component),
+			     ALLOW_IMAGES_MENU_PATH, allow_images_str);
+	gul_bonobo_set_label(BONOBO_UI_COMPONENT(window->ui_component),
+			     BLOCK_IMAGES_MENU_PATH, block_images_str);
+	gul_bonobo_set_label(BONOBO_UI_COMPONENT(window->ui_component),
+			     ALLOW_POPUPS_MENU_PATH, allow_popups_str);
+
+	g_free(allow_cookies_str);
+	g_free(block_cookies_str);
+	g_free(allow_images_str);
+	g_free(block_images_str);
+	g_free(allow_popups_str);
 }
 
 static void
@@ -1579,6 +1763,12 @@ update_favicon_control (GaleonWindow *window)
 static void
 update_find_control (GaleonWindow *window)
 {
+/* We're not updating the item sensitivity right now as it only follows the
+ * find dialog status. If you try to type-ahead-find with no valid search
+ * string in the find dialog, it won't let you Ctrl+G for the type-ahead-find
+ * even if there is a valid type-ahead-find result.
+ */
+#if MADE_TYPE_AHEAD_FIND_AWARE
 	gboolean can_go_next, can_go_prev;
 
 	if (window->priv->find_dialog)
@@ -1593,6 +1783,7 @@ update_find_control (GaleonWindow *window)
 		gul_bonobo_set_sensitive (BONOBO_UI_COMPONENT(window->ui_component),
 					  EDIT_FIND_PREV_CMD_PATH, can_go_prev);
 	}
+#endif
 }
 
 static void
@@ -1644,6 +1835,75 @@ update_spinner_control (GaleonWindow *window)
 	toolbar_spinner_stop (window->priv->toolbar);
 }
 
+static void
+update_permission_control (GaleonWindow *window)
+{
+	GaleonEmbedShell *embed_shell;
+	gresult result;
+	PermissionActionType image_action, cookie_action, popup_action;
+	GaleonTab *tab = galeon_window_get_active_tab(window);
+	if (tab == NULL) return;
+
+	embed_shell = galeon_shell_get_embed_shell(galeon_shell);
+	g_return_if_fail(embed_shell != NULL);
+
+	result = galeon_embed_shell_test_permission(embed_shell,
+						    galeon_tab_get_location(tab),
+						    IMAGES_PERMISSION,
+						    &image_action);
+	if (result == G_OK)
+	{
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 ALLOW_IMAGES_CMD_PATH,
+					 (image_action != ALLOW_ACTION));
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 BLOCK_IMAGES_CMD_PATH,
+					 (image_action != DENY_ACTION));
+	}
+
+	result = galeon_embed_shell_test_permission(embed_shell,
+						    galeon_tab_get_location(tab),
+						    COOKIES_PERMISSION,
+						    &cookie_action);
+	if (result == G_OK)
+	{
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 ALLOW_COOKIES_CMD_PATH,
+					 (cookie_action != ALLOW_ACTION));
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 BLOCK_COOKIES_CMD_PATH,
+					 (cookie_action != DENY_ACTION));
+	}
+
+	//If global allow popups, then the site specific menu item
+	//is always insensitive
+	if (eel_gconf_get_boolean(CONF_FILTERING_ALLOW_POPUPS))
+	{
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 ALLOW_POPUPS_CMD_PATH,
+					 FALSE);
+		galeon_tab_clear_blocked_popup_uri(tab);
+		update_popup_block(window);
+		return;
+	}
+	
+	result = galeon_embed_shell_test_permission(embed_shell,
+						    galeon_tab_get_location(tab),
+						    POPUPS_PERMISSION,
+						    &popup_action);
+	if (result == G_OK)
+	{
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 ALLOW_POPUPS_CMD_PATH,
+					 (popup_action != ALLOW_ACTION));
+		if (popup_action == ALLOW_ACTION)
+		{
+			galeon_tab_clear_blocked_popup_uri(tab);
+			update_popup_block(window);
+		}
+	}
+}
+
 void  
 galeon_window_update_control (GaleonWindow *window,
 			      ControlID control)
@@ -1661,6 +1921,9 @@ galeon_window_update_control (GaleonWindow *window,
 	case StatusbarSecurityControl:
 		update_security (window);
 		break;
+	case StatusbarPopupBlockControl:
+		update_popup_block(window);
+		break;
 	case FindControl:
 		update_find_control (window);
 		break;
@@ -1686,6 +1949,9 @@ galeon_window_update_control (GaleonWindow *window,
 	case FaviconControl:
 		update_favicon_control (window);
 		break;
+	case PermissionControl:
+		update_permission_control(window);
+		break;
 	default:
 		g_warning ("unknown control specified for updating");
 		break;
@@ -1706,8 +1972,10 @@ galeon_window_update_all_controls (GaleonWindow *window)
 		update_status_message (window);
 		update_progress (window);
 		update_security (window);
+		update_popup_block(window);
 		update_find_control (window);
 		update_spinner_control (window);
+		update_permission_control(window);
 	}
 }
 
@@ -1857,6 +2125,53 @@ galeon_window_notebook_switch_page_cb (GtkNotebook *notebook,
 	galeon_window_update_all_controls (window);
 }
 
+/**
+ * Prompt the user for a smart bookmark arguments, and return the
+ * completed URL to load, or NULL if cancelled.
+ */
+static char *
+handle_smartsite (GaleonWindow *w, GbSmartSite *s)
+{
+	GtkWidget *dialog, *vbox, *label, *entry;
+	int response;
+	char *args[2]; /* TODO: match this with get_num_fields() */
+	char *url;
+	
+	dialog = gtk_dialog_new_with_buttons (_("Galeon"), GTK_WINDOW(w),
+					      GTK_DIALOG_MODAL,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+					      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+					      NULL);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+
+	vbox = gtk_vbox_new (FALSE, 8);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
+
+	label = gtk_label_new (_("Enter the arguments for this smart site:"));
+	gtk_box_pack_start_defaults (GTK_BOX (vbox), label);
+
+	entry = gtk_entry_new ();
+	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+	gtk_box_pack_start_defaults (GTK_BOX (vbox), entry);
+
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
+	gtk_widget_show_all (dialog);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (response != GTK_RESPONSE_ACCEPT) {
+		gtk_widget_destroy (dialog);
+		return NULL;
+	}
+
+	args[0] = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
+	args[1] = NULL;
+	gtk_widget_destroy (dialog);
+	url = gb_smart_site_subst_args(s, args);
+	g_free(args[0]);
+	return url;
+}
+
 /**
  * Handle the activation of a bookmark. This function is not static because it's 
  * used in galeon-window-bookmark-activated-proxy.c (and only there: it's not public)
@@ -1866,13 +2181,23 @@ galeon_window_bookmark_activate (GaleonWindow *w, GbBookmarkEventActivated *ev)
 {
 	GaleonTab *tab = galeon_window_get_active_tab (w);
 	GbBookmark *b = ev->bookmark;
-	const char *url = ev->url;
+	char *url = g_strdup (ev->url);
 	GbBookmarkActivatedFlags f = ev->flags;
 	GaleonNewTabFlags flags = 0;
 
+	if (GB_IS_SMART_SITE(b) && GB_SMART_SITE(b)->smarturl != NULL &&
+	    (GB_SITE(b)->url == NULL || GB_SITE(b)->url[0]=='\0')) {
+		url = handle_smartsite (w, GB_SMART_SITE (b));
+		if (url == NULL)
+		{
+			return;
+		}
+	}
+
 	if (GB_IS_SITE (b) && f == GB_BAF_DEFAULT)
 	{
 		galeon_window_load_url (w, url);
+		g_free (url);
 		return;
 	}
 
@@ -1904,51 +2229,91 @@ galeon_window_bookmark_activate (GaleonWindow *w, GbBookmarkEventActivated *ev)
 	{
 		galeon_shell_new_tab (galeon_shell, w, tab, url, flags);
 	}
-	else if (GB_IS_FOLDER (b) && flags & GALEON_NEW_TAB_IN_NEW_WINDOW)
-	{
-		GbBookmark *bi;
-		for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
-		{
-			if (GB_IS_SITE (bi))
-			{
-				galeon_shell_new_tab (galeon_shell, w, tab, 
-						      GB_SITE (bi)->url, flags);
-			}
-		}
-	}
 	else if (GB_IS_FOLDER (b))
 	{
 		GbBookmark *bi;
-		GaleonTab *jump_to_tab;
-		gboolean jump_to;
+		int count = 0;
 		
-		jump_to = flags & GALEON_NEW_TAB_JUMP;
-		if (flags & GALEON_NEW_TAB_DONT_JUMP_TO) jump_to = FALSE;
-
-		/* Open all bookmarks in order in the background.  Jumping
-		 * to them automatically is a major performance hit with
-		 * >3 bookmarks in a folder.  Once all bookmarks are opened,
-		 * we can efficiently jump to the first one if 'jump to
-		 * new tabs automatically' is enabled.
-		 */
-		flags |= GALEON_NEW_TAB_DONT_JUMP_TO;
-
-		jump_to_tab = NULL;
+		/* count the number of activated bookmarks */
 		for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
 		{
 			if (GB_IS_SITE (bi))
 			{
-				tab = galeon_shell_new_tab (galeon_shell, w, tab,
-							    GB_SITE (bi)->url, 
-							    flags);
-
-				if (jump_to_tab == NULL) jump_to_tab = tab;
+				count++;
 			}
 		}
-
-		if (jump_to && jump_to_tab != NULL)
+		
+#define MANY_BOOKMARKS_WARNING_THRESHOLD 5
+		if (count > MANY_BOOKMARKS_WARNING_THRESHOLD)
 		{
-			galeon_window_jump_to_tab (w, jump_to_tab);
+			gint response;
+			GtkWidget *dialog = gtk_message_dialog_new 
+				(GTK_WINDOW (w),
+				 GTK_DIALOG_MODAL,
+				 GTK_MESSAGE_QUESTION,
+				 GTK_BUTTONS_NONE,
+				 _("You have requested to show %d bookmarked sites in different "
+				   "windows or tabs. Are you sure?"),
+				 count);
+
+			gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+			gtk_dialog_add_button (GTK_DIALOG (dialog), 
+					       _("Open the bookmarks"), 1);
+			gtk_dialog_set_default_response (GTK_DIALOG (dialog), 1);
+			
+			response = gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
+
+			if (response != 1)
+			{
+				return;
+			}
+		}
+		
+		if (flags & GALEON_NEW_TAB_IN_NEW_WINDOW)
+		{
+			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
+			{
+				if (GB_IS_SITE (bi))
+				{
+					galeon_shell_new_tab (galeon_shell, w, tab, 
+							      GB_SITE (bi)->url, flags);
+				}
+			}
+		}
+		else
+		{
+			GaleonTab *jump_to_tab;
+			gboolean jump_to;
+			
+			jump_to = flags & GALEON_NEW_TAB_JUMP;
+			if (flags & GALEON_NEW_TAB_DONT_JUMP_TO) jump_to = FALSE;
+			
+			/* Open all bookmarks in order in the background.  Jumping
+			 * to them automatically is a major performance hit with
+			 * >3 bookmarks in a folder.  Once all bookmarks are opened,
+			 * we can efficiently jump to the first one if 'jump to
+			 * new tabs automatically' is enabled.
+			 */
+			flags |= GALEON_NEW_TAB_DONT_JUMP_TO;
+			
+			jump_to_tab = NULL;
+			for (bi = GB_FOLDER (b)->child; bi; bi = bi->next)
+			{
+				if (GB_IS_SITE (bi))
+				{
+					tab = galeon_shell_new_tab (galeon_shell, w, tab,
+								    GB_SITE (bi)->url, 
+								    flags);
+					
+					if (jump_to_tab == NULL) jump_to_tab = tab;
+				}
+			}
+
+			if (jump_to && jump_to_tab != NULL)
+			{
+				galeon_window_jump_to_tab (w, jump_to_tab);
+			}
 		}
 	}
 	else
