@@ -254,9 +254,11 @@ crashed_resume_dialog (Session *session)
  *
  * Resume a crashed session when necessary (interactive)
  * 
- * Return value: return false if no window  has been opened
+ * Return value: returns SESSION_ABORT   if the user wants to abort starting galeon
+ *                       SESSION_RESUMED if a session has been resumed
+ *                       SESSION_OK      if no session was resumed
  **/
-gboolean
+SessionResumeType
 session_autoresume (Session *session)
 {
 	char *saved_session;
@@ -266,13 +268,13 @@ session_autoresume (Session *session)
 
 	if (g_file_test (saved_session, G_FILE_TEST_EXISTS))
 	{
-		/* we want to exit now, so dont open windows
-		 * but return like if we did . Also we need
-		 * to not remove the crashed session. */
+		/* we want to exit now, so tell our caller that the
+		 * user wants to abort, but dont remove the crashed
+		 * session. */
 		if (!crashed_resume_dialog (session))
 		{
 			session->priv->dont_remove_crashed = TRUE;
-			return TRUE;
+			return SESSION_ABORT;
 		}
 			
 		loaded = TRUE;
@@ -291,8 +293,8 @@ session_autoresume (Session *session)
 	
 	g_free (saved_session);
 
-	/* return false if no window has been opened */
-	return (session->priv->windows != NULL);
+	/* return SESSION_RESUMED, if a window was opened, else SESSION_OK */
+	return (session->priv->windows != NULL) ? SESSION_RESUMED : SESSION_OK;
 }
 
 static void
