@@ -42,6 +42,7 @@
 #include "galeon-home-button.h"
 #include "galeon-new-button.h"
 #include "galeon-stop-button.h"
+#include "galeon-fullscreen-button.h"
 
 #include <string.h>
 #include <bonobo/bonobo-i18n.h>
@@ -54,11 +55,13 @@
 
 #define DEFAULT_TOOLBAR_SETUP \
 	"back_menu=navigation_button(direction=back,arrow=TRUE);" \
-	"up_menu=navigation_button(direction=up,arrow=TRUE);" \
 	"forward_menu=navigation_button(direction=forward,arrow=TRUE);" \
 	"stop=stop_button;" \
 	"reload=reload_button;" \
+	"home_separator=separator;" \
 	"home=home_button;" \
+	"fullscreen=fullscreen_button;" \
+	"favicons_separator=separator;" \
 	"favicon=favicon;" \
 	"location=location;" \
 	"zoom=zoom;" \
@@ -123,6 +126,7 @@ struct ToolbarPrivate
 	GtkWidget *zoom_spinbutton;
 	guint zoom_timeout_id;
 	gboolean zoom_lock;
+	GaleonFullscreenButton *fullscreen_button;
 };
 
 GType 
@@ -180,6 +184,7 @@ toolbar_class_init (ToolbarClass *klass)
 	gul_toolbar_item_register_type ("home_button", (GulTbItemConstructor) galeon_home_button_new);
 	gul_toolbar_item_register_type ("stop_button", (GulTbItemConstructor) galeon_stop_button_new);
 	gul_toolbar_item_register_type ("new_button", (GulTbItemConstructor) galeon_new_button_new);
+	gul_toolbar_item_register_type ("fullscreen_button", (GulTbItemConstructor) galeon_fullscreen_button_new);
 }
 
 static void
@@ -215,9 +220,29 @@ toolbar_get_property (GObject *object,
 }
 
 static void
-toolbar_location_url_activate_cb (GaleonLocationEntry *entry, const gchar *url, GaleonWindow *window)
+toolbar_location_url_activate_cb (GaleonLocationEntry *entry, const gchar *url,
+                                  gboolean new_tab_or_window, GaleonWindow *window)
 {
-	galeon_window_load_url (window, url);
+	GbBookmarkSet *set;
+	gchar *real_url;
+	GaleonTab *tab;
+
+	if (!new_tab_or_window)
+	{
+		/* galeon_window_load_url handles bookmark nicks */
+		galeon_window_load_url (window, url);
+		return;
+	}
+
+	set = galeon_shell_get_bookmark_set (galeon_shell);
+	real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
+
+	/* Open the link in a new tab (or window) */
+	tab = galeon_window_get_active_tab (window);
+	galeon_shell_new_tab (galeon_shell, window, tab, 
+			      real_url != NULL ? real_url : url, 0);
+	
+	g_free (real_url);
 }
 
 static void
@@ -374,9 +399,8 @@ toolbar_setup_style (Toolbar *t)
 			style = styles[i];
 		}
 		gconf_value_free (value);
+		gul_bonobo_set_look (t->priv->ui_component, "/Toolbar", style);
 	}
-
-	gul_bonobo_set_look (t->priv->ui_component, "/Toolbar", style);
 }
 
 static void
@@ -443,6 +467,12 @@ toolbar_get_widgets (Toolbar *t)
 	}
 	p->go_button = NULL;
 
+	if (p->fullscreen_button)
+	{
+		g_object_unref (p->fullscreen_button);
+	}
+	p->fullscreen_button = NULL;
+
 	if (p->home_button)
 	{
 		g_object_unref (p->home_button);
@@ -610,6 +640,25 @@ toolbar_get_widgets (Toolbar *t)
 		}
 	}
 
+	it = gul_toolbar_get_item_by_id (gt, "fullscreen");
+	if (it)
+	{
+		if (GALEON_IS_FULLSCREEN_BUTTON (it))
+		{	
+			DEBUG_MSG (("    got a fullscreen button\n"));
+			p->fullscreen_button = g_object_ref (it);
+			if (p->window)
+			{
+				galeon_tbi_set_window (GALEON_TBI (it), p->window);
+			}
+		}
+		else
+		{
+			g_warning ("An unexpected button has been found in your toolbar. "
+				   "Maybe your setup is too old.");
+		}
+	}
+
 	it = gul_toolbar_get_item_by_id (gt, "location");
 	if (it)
 	{
@@ -753,6 +802,11 @@ toolbar_finalize (GObject *object)
 		g_object_unref (p->new_button);
 	}
 
+	if (t->priv->fullscreen_button)
+	{
+		g_object_unref (p->fullscreen_button);
+	}
+
         g_free (t->priv);
 
 	DEBUG_MSG (("Toolbar finalized\n"));
@@ -787,11 +841,11 @@ toolbar_set_visibility (Toolbar *t, gboolean visibility)
 }
 
 void 
-toolbar_activate_location (Toolbar *t)
+toolbar_edit_location (Toolbar *t)
 {
 	if (t->priv->location_entry)
 	{
-		galeon_location_entry_activate 
+		galeon_location_entry_edit 
 			(GALEON_LOCATION_ENTRY(t->priv->location_entry));
 	}
 }
@@ -869,6 +923,17 @@ toolbar_set_location (Toolbar *t,
 	}
 }
 
+void
+toolbar_set_fullscreen (Toolbar *t, gboolean s)
+{
+	if (t->priv->fullscreen_button)
+	{
+		galeon_fullscreen_button_set_fullscreen 
+			( GALEON_FULLSCREEN_BUTTON(t->priv->fullscreen_button), s);
+	}
+}
+
+
 void
 toolbar_update_favicon (Toolbar *t)
 {
