@@ -24,6 +24,7 @@
  */
 
 #include "gul-notebook.h"
+#include "gul-gui.h"
 #include "galeon-marshal.h"
 
 #include <gtk/gtk.h>
@@ -626,18 +627,15 @@ gul_notebook_finalize (GObject *object)
 
 	notebooks = g_list_remove (notebooks, notebook);
 
-	if (notebook->priv->focused_pages)
-	{
-		g_list_free (notebook->priv->focused_pages);
-	}
+	g_list_free (notebook->priv->focused_pages);
 	g_list_free (notebook->priv->opened_tabs);
-	
+
 	g_free (notebook->priv);
 }
 
 static void
-galeon_tab_close_button_clicked_cb (GtkWidget *widget,
-				    GtkWidget *child)
+tab_close_button_clicked_cb (GtkWidget *widget,
+			     GtkWidget *child)
 {
 	GulNotebook *notebook;
 
@@ -645,32 +643,47 @@ galeon_tab_close_button_clicked_cb (GtkWidget *widget,
 	gul_notebook_remove_page (notebook, child);
 }
 
+static void
+tab_close_button_state_changed_cb (GtkWidget	*widget,
+		                   GtkStateType	 previous_state)
+{
+	if (GTK_WIDGET_STATE(widget) == GTK_STATE_ACTIVE)
+	{
+		gtk_widget_set_state (widget, GTK_STATE_NORMAL);
+	}
+}
+
 static GtkWidget *
 tab_build_label (GulNotebook *nb, GtkWidget *child)
 {
 	GtkWidget *label, *hbox, *close_button, *image;
-	int h, w;
+	GtkRequisition size;
+	GtkRcStyle *rcstyle;
 	GClosure *closure;
 	GtkWidget *window;
 
 	window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
 	
-	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
-	
 	/* set hbox spacing and label padding (see below) so that there's an
 	 * equal amount of space around the label */
 	hbox = gtk_hbox_new (FALSE, 4);
 
-	/* setup close button */
+	/* setup close button, zero out {x,y}thickness to get smallest possible
+	 * size */
 	close_button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (close_button), 
 			       GTK_RELIEF_NONE);
+	rcstyle = gtk_rc_style_new ();
+	rcstyle->xthickness = rcstyle->ythickness = 0;
+	gtk_widget_modify_style (close_button, rcstyle);
+	gtk_rc_style_unref (rcstyle),
+
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
 					  GTK_ICON_SIZE_MENU);
-	gtk_widget_set_size_request (close_button, w+2, h+2);
-	gtk_container_add (GTK_CONTAINER (close_button), 
-			   image);
-		
+	gtk_widget_size_request (image, &size);
+	gtk_widget_set_size_request (close_button, size.width, size.height);
+	gtk_container_add (GTK_CONTAINER (close_button), image);
+
 	/* setup label */ 
         label = gtk_label_new (_("Untitled"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
@@ -693,7 +706,14 @@ tab_build_label (GulNotebook *nb, GtkWidget *child)
                             FALSE, FALSE, 0);
 
 	g_signal_connect (G_OBJECT (close_button), "clicked",
-                          G_CALLBACK (galeon_tab_close_button_clicked_cb),
+                          G_CALLBACK (tab_close_button_clicked_cb),
+                          child);
+
+	/* work around a cosmetic bug in gtk where buttons with GTK_RELIEF_NONE
+	 * have annoyingly visible border only when in GTK_STATE_ACTIVE (ie. in
+	 * inactive tabs) */
+	g_signal_connect (G_OBJECT (close_button), "state_changed",
+                          G_CALLBACK (tab_close_button_state_changed_cb),
                           child);
 		
 	gtk_widget_show (hbox);
@@ -814,11 +834,12 @@ gul_notebook_remove_page (GulNotebook *nb,
 	int position, cur;
 	gboolean last_tab;
 
-	last_tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (nb), 1) == NULL;
+	last_tab = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb)) == 1;
 	if (last_tab)
 	{
 		GtkWidget *window;
 		window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
+		gul_gui_widget_hide_now (window);
 		gtk_widget_destroy (window);
 		return;
 	}
